@@ -17,6 +17,9 @@ def default(val, d):
 def is_power_two(n):
     return math.log2(n).is_integer()
 
+def l2norm(t):
+    return F.normalize(t, dim = -1)
+
 # helper classes
 
 def Upsample(dim):
@@ -100,10 +103,11 @@ class Attention(nn.Module):
         self,
         dim,
         heads = 4,
-        dim_head = 64
+        dim_head = 64,
+        scale = 16
     ):
         super().__init__()
-        self.scale = dim_head ** -0.5
+        self.scale = scale
         self.heads = heads
         inner_dim = heads * dim_head
         self.norm = LayerNorm(dim)
@@ -119,9 +123,9 @@ class Attention(nn.Module):
         q, k, v = self.to_qkv(x).chunk(3, dim = 1)
         q, k, v = map(lambda t: rearrange(t, 'b (h c) ... -> b h (...) c', h = self.heads), (q, k, v))
 
-        q = q * self.scale
+        q, k = map(l2norm, (q, k))
 
-        sim = einsum('b h i d, b h j d -> b h i j', q, k)
+        sim = einsum('b h i d, b h j d -> b h i j', q, k) * self.scale
         attn = sim.softmax(dim = -1)
 
         out = einsum('b h i j, b h j d -> b h i d', attn, v)
